require 'json'
require 'net/https'
require 'cgi/cookie'
require 'pp'
require 'uri'
require 'optparse'


module ZephyrClient::Utils

  extend self

  def self.createRequest(user, http_method='get', path=nil, body=nil, headers={})
    request = Net::HTTP.const_get(http_method.to_s.capitalize).new(path, headers)
    request.body = body unless body.nil?
    request.basic_auth(user[:username], user[:password])
    request
  end


  def self.connect(uri, options)
    #puts __FILE__ + (__LINE__).to_s + " [ Utils.connect( #{uri} ) ]"

    targetUri = URI.parse(uri)
    http_class = Net::HTTP
    httpReq = http_class.new(targetUri.host, targetUri.port)
    httpReq.use_ssl = options['use_ssl']
    httpReq.verify_mode = options['ssl_verify_mode']
    httpReq.read_timeout = options['read_timeout']

    #puts __FILE__ + (__LINE__).to_s + " => #{httpReq}"

    httpReq
  end

  def self.parseCommandLine(args=nil)
    options={ :verbose => false }

    opt_parser = OptionParser.new do |opt|

      opt.on('-c', '--cycle Cycle') { |o|
        options[:cycle] = o
      }

      opt.on('--cfg', '--cfg Config') { |o|
        options[:config] = o
      }

      opt.on('-r', '--release Release') { |o|
        options[:release] = o
      }

      opt.on('-p', '--project ProjectKey') { |o|
        options[:project] = o
      }

      opt.on('--password', '--password Password') { |o|
        options[:password] = o
      }
      opt.on('u', '--user UserId') { |o|
        options[:user] = o
      }
      opt.on('--jira', '--jira JIRA') { |o|
        options[:jira] = o
      }

      opt.on('--status', '--status Status') { |o|
        options[:status] = o
      }

      opt.on('-v', '--verbose') { |o|
        options[:verbose] = o
      }

    end


    if !args.nil?
      opt_parser.parse!(args)
    else
      opt_parser.parse!
    end

    @options=options
  end

  def self.isVerbose()
    @options[:verbose]
  end

end
